#ifndef _UTILITY_H_
#define _UTILITY_H_

#ifdef __cplusplus
#include <cstring>
#include <cstdlib>
#else
#include <string.h>
#include <stdlib.h>
#endif
//!note: io_png assumes deinterleaved image buffer (ie R1,R2,...,G1,G2,...,B1,B2,...) while iio provides interleaved data (R1,G1,B1,R2,G2,B2,...)
static inline void
deinterleave(float* input, float* output, int w, int h, int pd)
{
    for (int channel=0; channel<pd; channel++)
    {
        float* ch=output+channel*w*h;
        // Copy the current channel
        for (int pix=0; pix<w*h; pix++)
            ch[pix] = input[pd*pix+channel];
    }
}

static inline void
interleave(float* input, float* output, int w, int h, int pd)
{
    for (int channel=0; channel<pd; channel++)
    {
        float* ch=input+channel*w*h;
        // Copy the current channel
        for (int pix=0; pix<w*h; pix++)
            output[pd*pix+channel]=ch[pix];
    }
}


// note the inplace version are not more efficient. Merely, they have a simpler api
static inline void
deinterleaveInPlace(float* input, int w, int h, int pd)
{
    float* tmp=(float*)malloc(w*h*pd*sizeof(float));
    memcpy((void*)tmp, (void*)input, w*h*pd*sizeof(float));
    deinterleave(tmp, input, w, h, pd);
    free(tmp);
}

static inline void
interleaveInPlace(float* input, int w, int h, int pd)
{
    float* tmp=(float*)malloc(w*h*pd*sizeof(float));
    memcpy((void*)tmp, (void*)input, w*h*pd*sizeof(float));
    interleave(tmp, input, w, h, pd);
    free(tmp);
}

#endif /* _UTILITY_H_ */
