#include <cstdlib>
#include <string>
#include <iostream>
#include <fstream>

extern "C"
{
#include "iio.h"
}
#include "interpolation/interpolationAPI.h"

using namespace ipol;


float truncate(float val, float th)
{
    val *= 128./th;
    val += 128.;
    val= static_cast<int>(val + .5);
    return (val > 255 ? 255. : (val < 0 ? 0. : val));
}

int main(int argc, char *argv[])
{
    // check command line arguments
    if (argc < 4 || argc > 5 ) {
        std::cerr<<"usage:\n\t"<<argv[0]<<"in1 in2 th [out]"<<std::endl;
        //                          0       1   2  3    4 
        return EXIT_FAILURE;
    }
    std::string inName1= (argc>1)? argv[1] : "-";
    std::string inName2= (argc>2)? argv[2] : "-";
    float th=atof(argv[3]);
    std::string outName= (argc>3)? argv[4] : "-";
    // read input image "im"
    int w1,h1,pd1;
    float *buf1 = static_cast<float*>(iio_read_image_float_split(inName1.c_str(), &w1, &h1, &pd1));
    int w2,h2,pd2;
    float *buf2 = static_cast<float*>(iio_read_image_float_split(inName2.c_str(), &w2, &h2, &pd2));
    if ((w1 != w2) || (h1 != h2) || (pd1 != pd2))
    {
        std::cerr<<"image sizes don't match"<<std::endl;
        free(buf1);
        free(buf2);
        return EXIT_FAILURE;
    }
    ImageSize size(w1,h1,pd1);
    ImageF im1(buf1, size);
    ImageF im2(buf2, size);
    // Compute the Mean Square Error between buf1 and buf2
    ImageF imDiff=im1-im2;
    for(int k=0; k < imDiff.getNumPixels()*size.m_nc; k++)
    {
        imDiff[k]=truncate(imDiff[k],th);
    }

    iio_save_image_float_split(const_cast<char*>(outName.c_str()), imDiff.getData(), w1, h1, pd1);
    // free the memory allocated by iio
    free(buf1);
    free(buf2);
    return 0;
}
