#include <cmath>

#include "common/utility.h"
#include "interpolation/interpolationAPI.h"
#include <cmath>
using namespace ipol;

//////// InterpolatedImageF
InterpolatedImageF::InterpolatedImageF()
{
}

InterpolatedImageF::~InterpolatedImageF()
{
}


void
InterpolatedImageF::setImage(const ImageF& image)
{
    m_image=image.clone();
}


ImageF
InterpolatedImageF::onSeparableGrid(SmartArray<float> xs, SmartArray<float> ys, int c) const
{
    //!fixme not tested yet
    int nc=m_image.getSize().m_nc;
    if(c>=0)
    {
        nc=1;
    }
    ImageSize size(xs.getLength(), ys.getLength(),nc);
    SmartArray<float> xs2(size.m_width*size.m_height), ys2(size.m_width*size.m_height);
    for (int i = 0; i < size.m_width; ++i)
    {
        for (int j = 0; j < size.m_height; ++j)
        {
            xs2[i+j*size.m_width]=xs[i];
            ys2[i+j*size.m_width]=ys[j];
        }
    }
    SmartArray<float> buf=(*this)(xs2,ys2,c);
    return ImageF(buf,size);
}

//////// ImageSize
ImageSize::ImageSize(int width, int height, int nc):m_width(width),m_height(height),m_nc(nc)
{
}





/////// utility routines
void
ipol::makeScaleRotationGrid(SmartArray<float>& xs, SmartArray<float>& ys, int& outWidth, int& outHeight, int width, int height, float Scale, float Theta)
{
    const int ScaleWidth = floor(Scale*width + 0.5f);
    const int ScaleHeight = floor(Scale*height + 0.5f);
    const float x0 = (float)ScaleWidth/2.0f;
    const float y0 = (float)ScaleHeight/2.0f;
    const float CosTheta = (float)cos(Theta);
    const float SinTheta = (float)sin(Theta);
    float CurX, CurY, XStart, YStart, *XPtr, *YPtr;
    int m, n;

     /* Determine the support of the transformed image. */
    XStart = -fabs(CosTheta)*x0 - fabs(SinTheta)*y0;
    YStart = -fabs(SinTheta)*x0 - fabs(CosTheta)*y0;
    outWidth = floor(ScaleWidth*fabs(CosTheta)
        + ScaleHeight*fabs(SinTheta) + 0.5f);
    outHeight = floor(ScaleWidth*fabs(SinTheta)
        + ScaleHeight*fabs(CosTheta) + 0.5f);
    int numentries=outWidth*outHeight;
    xs.reallocate(numentries);
    ys.reallocate(numentries);

    /* Create the sampling grid */
    XPtr=xs.get();
    YPtr=ys.get();
    for(n = 0; n < outHeight; n++)
        for(m = 0; m < outWidth; m++)
        {
            CurX = XStart + m;
            CurY = YStart + n;
            *(XPtr++) = (x0 + CosTheta*CurX - SinTheta*CurY) / Scale;
            *(YPtr++) = (y0 + SinTheta*CurX + CosTheta*CurY) / Scale;
        }
}


void
ipol::makeScaleSeparableGrid(SmartArray<float>& xs, SmartArray<float>& ys, int& outWidth, int& outHeight, int width, int height, float scale)
{
    outWidth = ceil(width * scale);
    outHeight = ceil(height * scale);
    float xstep = (1.0 * width) / outWidth;
    float ystep = (1.0 * height) / outHeight;
    float xstart = (xstep - 1.0)/2.0;
    float ystart = (ystep - 1.0)/2.0;
    xs.reallocate(outWidth);
    ys.reallocate(outHeight);
    for(int xpos=0; xpos<(int)outWidth; xpos++)
        xs[xpos]=xstart + xstep*xpos;
    for(int ypos=0; ypos<(int)outHeight; ypos++)
        ys[ypos]=ystart + ystep*ypos;
}


void
ipol::makeRegularSeparableGrid(SmartArray<float>& xs, SmartArray<float>& ys, int& outWidth, int& outHeight, float xstart, float ystart, float xend, float yend, float xstep, float ystep)
{
    outWidth = ceil((xend-xstart)/xstep);
    outHeight = ceil((yend-ystart)/ystep);
    xs.reallocate(outWidth);
    ys.reallocate(outHeight);
    for(int xpos=0; xpos<outWidth; xpos++)
        xs[xpos]=xstart + xstep*xpos;
    for(int ypos=0; ypos<outHeight; ypos++)
        ys[ypos]=ystart + ystep*ypos;
}


SmartArray<float>
ipol::makeRange(float first, float last, float step, bool included)
{
    int nb=static_cast<int>( (last - first)/step );
    if(included)
        nb+=1;
    SmartArray<float> range(nb);
    float lastelem=first;
    for (int i = 0; i < nb; ++i)
    {
        range[i]=lastelem;
        lastelem+=step;
    }
    return range;
}
