
#ifndef _INTERPOLATIONAPI_H_
#define _INTERPOLATIONAPI_H_
#include <string>
#include <cassert>


#include "common/image.h"

namespace ipol
{


   // a class to wrap interpolation capabilities:
    //
    //
    class InterpolatedImageF
    {
    public:
        InterpolatedImageF();
        virtual ~InterpolatedImageF();

        virtual float operator()(float x, float y, int c=0) const=0;
        
        // if c==-1 returns all layers and only layer c otherwise
        // xs and ys should have the same length
        virtual SmartArray<float> operator()(SmartArray<float> xs, SmartArray<float> ys, int c=-1) const=0;

        // if c==-1 returns all layers and only layer c otherwise
        virtual ImageF onSeparableGrid(SmartArray<float> xs, SmartArray<float> ys, int c=-1) const;

        virtual void setImage(const ImageF& image);// clone it

    protected:
        ImageF m_image;
    };

    //////// utility routines
    void makeScaleRotationGrid(SmartArray<float>& xs, SmartArray<float>& ys, int& outWidth, int& outHeight, int width, int height, float Scale, float Theta);
    void makeScaleSeparableGrid(SmartArray<float>& xs, SmartArray<float>& ys, int& outWidth, int& outHeight, int width, int height, float scale);
    void makeRegularSeparableGrid(SmartArray<float>& xs, SmartArray<float>& ys, int& outWidth, int& outHeight, float xstart, float ystart, float xend, float yend, float xstep=1, float ystep=1);
    SmartArray<float> makeRange(float first, float last, float step=1., bool included=true);
}

#endif /* _INTERPOLATIONAPI_H_ */
