#include <cstdlib>
#include <string>
#include <iostream>
#include <fstream>

extern "C"
{
#include "iio.h"
}
#include "interpolation/interpolationAPI.h"

using namespace ipol;

int main(int argc, char *argv[])
{
    // check command line arguments
    if (argc < 4 || argc > 5 ) {
        std::cerr<<"usage:\n\t"<<argv[0]<<"in1 in2 out"<<std::endl;
        //                          0       1   2   3
        return EXIT_FAILURE;
    }
    std::string inName1= (argc>1)? argv[1] : "-";
    std::string inName2= (argc>2)? argv[2] : "-";
    std::string outName= (argc>3)? argv[3] : "-";
    // read input image "im"
    int w1,h1,pd1;
    float *buf1 = static_cast<float*>(iio_read_image_float_split(inName1.c_str(), &w1, &h1, &pd1));
    int w2,h2,pd2;
    float *buf2 = static_cast<float*>(iio_read_image_float_split(inName2.c_str(), &w2, &h2, &pd2));
    if ((w1 != w2) || (h1 != h2) || (pd1 != pd2))
    {
        std::cerr<<"image sizes don't match"<<std::endl;
        free(buf1);
        free(buf2);
        return EXIT_FAILURE;
    }

    ImageSize size(w1,h1,pd1);
    ImageF im1(buf1, size);
    ImageF im2(buf2, size);
    // Compute the Mean Squared Error between buf1 and buf2
    ImageF imDiff=im1-im2;
    ImageF imDiff2=imDiff.pow(2);
    // the mean is taken along y axis and channels and symmetrically (that is
    // entries at the same distance from the vertical borders are averaged)
    SmartArray<float> meanSquareError(size.m_width/2);
    for(int x=0; x < size.m_width/2; x++)
    {
        meanSquareError[x]=0;
        for(int y=0; y < size.m_height; y++)
            for(int c=0; c < size.m_nc; c++)
            {
                meanSquareError[x]+=imDiff2(x,y,c);
                meanSquareError[x]+=imDiff2(size.m_width-1-x,y,c);
            }
        meanSquareError[x]/=size.m_height*size.m_nc*2.;
    }
    //dump the meanSquareError entries in a file
    std::ofstream sout(outName.c_str());
    for(int x=0; x<size.m_width/2; x++)
    {
        sout<<x<<" "<<meanSquareError[x]<<std::endl;
    }
    sout.close();
    // free the memory allocated by iio
    free(buf1);
    free(buf2);
    return 0;
}
