#include <cstdlib>
#include <string>
#include <iostream>
extern "C"
{
#include "iio.h"
}
#include "common/utility.h"
#include "interpolation/interpolationAPI.h"
#include "interpolation/linearInterpolation.h"

using namespace ipol;

void createSampleLocations(char *argv[], int w, int h, SmartArray<float>& xs, SmartArray<float>& ys)
{
    float dx=atof(argv[3]);
    float dy=atof(argv[4]);
    float x1=atof(argv[5]);
    float y1=atof(argv[6]);
    float x2=atof(argv[7]);
    float y2=atof(argv[8]);
    if(x2<0) x2=w-1;
    if(y2<0) y2=h-1;
    xs=makeRange(x1+dx, x2+dx, 1, true);
    ys=makeRange(y1+dy, y2+dy, 1, true);
}

int main(int argc, char *argv[])
{
    // check command line arguments
    if (argc < 9 || argc > 11 ) {
        std::cerr<<"usage:\n\t"<<argv[0]<<" method variant dx dy x1 y1 x2 y2 [in [out]]"<<std::endl;
        //                          0          1      2    3  4  5  6  7  8   9    10
        return EXIT_FAILURE;
    }
    else
    {
        std::cerr<<argv[0]<<" "<<argv[1]<<" "<<argv[2]<<" "<<argv[3]<<" "<<argv[4]<<" "<<argv[5]<<" "<<argv[6]<<" "<<argv[7]<<" "<<argv[8]<<std::endl; 
    }
    std::string methodName=argv[1];
    std::string variant=argv[2];
    std::string inName= (argc>9)? argv[9] : "-";
    std::string outName= (argc>10)? argv[10] : "-";
    // read input image "im"
    int w,h,pd;
    float *buf = static_cast<float*>(iio_read_image_float_split(inName.c_str(), &w, &h, &pd));
    ImageSize size(w,h,pd);
    bool interleaved=false; // take care that iio returns an interleaved buffer while ImageF requires a de-interleaved one.
    ImageF im(buf, size, interleaved);
    free(buf);
    // produce the interpolation grid
    SmartArray<float> xs;
    SmartArray<float> ys;
    int axis=-1;
    createSampleLocations(argv, w, h, xs, ys);
    InterpolatedImageF* intim;
    if(methodName=="shannon")
    {   
        bool periodic=(variant=="periodic");
        bool centered=(variant=="centered")||(variant=="acc");
        bool pps=(variant=="pps");
        int accIter=0;
        //!fixme: there seems to be no gain for acc>1
        //!fixme: Note that acc=1, probably corresponds to taking Cesaro sum instead of direct summation
        if(variant=="acc")
            accIter=1;
        intim=new SincInterpolatedImageF(periodic, centered, pps, accIter, axis);
    }
    else
    {
        intim= new LinearInterpolatedImageF(methodName,variant);
    }
    intim->setImage(im);
    // save output image
    ImageF imZ = intim->onSeparableGrid(xs,ys);
    delete intim;
    ImageSize outSize=imZ.getSize();
    int out_w=outSize.m_width, out_h=outSize.m_height;
    int out_pd=outSize.m_nc;
    iio_save_image_float_split(const_cast<char*>(outName.c_str()), imZ.getData(), out_w, out_h, out_pd);
    return 0;
}
